// framedraw.c
// LiVES
// (c) G. Finch (salsaman@xs4all.nl)
// see file ../COPYING for licensing details

// functions for the 'framedraw' widget - lets users draw on frames :-)

#include "main.h"
#include "framedraw.h"
#include "callbacks.h"
#include "support.h"
#include "interface.h"

static gint xstart,ystart;
static gboolean b1_held;



void  
widget_add_framedraw (void *top, GtkBox *box, gint start, gint end, gboolean has_redraw_button) {
  // adds the frame draw widget to box
  // the redraw button should be connected to an appropriate redraw function
  // after calling this function

  // an example of this is in 'trim frames'

  GtkWidget *vseparator7;
  GtkWidget *vbox27;
  GtkWidget *hbox27;
  GtkWidget *label108;
  GtkObject *spinbutton53_adj;
  GtkWidget *spinbutton53;
  GtkWidget *label114;
  GtkWidget *framedraw_scale;
  gint hsize,vsize;
  GtkWidget *frame;

  b1_held=FALSE;

  vseparator7 = gtk_vseparator_new ();
  gtk_widget_show (vseparator7);
  gtk_box_pack_start (GTK_BOX (box), vseparator7, TRUE, TRUE, 0);

  vbox27 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox27);
  gtk_box_pack_start (GTK_BOX (box), vbox27, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (vbox27), 10);

  frame = gtk_frame_new (NULL);
  gtk_widget_show (frame);
  gtk_box_pack_start (GTK_BOX (vbox27), frame, TRUE, TRUE, 0);
  gtk_widget_modify_bg (frame, GTK_STATE_NORMAL, &palette->normal_back);

  label108 = gtk_label_new (_("Preview"));
  gtk_widget_show (label108);
  gtk_frame_set_label_widget (GTK_FRAME (frame), label108);
  gtk_label_set_justify (GTK_LABEL (label108), GTK_JUSTIFY_LEFT);
  if (palette->style&STYLE_1) {
    gtk_widget_modify_fg(label108, GTK_STATE_NORMAL, &palette->normal_fore);
  }

  hsize=cfile->hsize;
  vsize=cfile->vsize;
  if (hsize<58) {
    hsize=58;
  }
  if (vsize<32) {
    vsize=32;
  }

  gtk_frame_set_shadow_type (GTK_FRAME(frame), GTK_SHADOW_IN);
  mainw->framedraw=gtk_event_box_new();
  gtk_widget_set_size_request (mainw->framedraw, hsize, vsize);
  gtk_widget_modify_bg (mainw->framedraw, GTK_STATE_NORMAL, &palette->normal_back);
  gtk_widget_set_events (mainw->framedraw, GDK_BUTTON1_MOTION_MASK | GDK_BUTTON_RELEASE | GDK_BUTTON_PRESS);

  mainw->framedraw_frame=start;

  // create a blank bitmap mask
  if (!(mainw->framedraw_bitmap==NULL)) {
    gdk_pixmap_unref (mainw->framedraw_bitmap);
  }
  mainw->framedraw_bitmap = gdk_pixmap_new (NULL, cfile->hsize, cfile->vsize, 1);

  if (!(mainw->framedraw_bitmapgc==NULL)) {
    g_object_unref (mainw->framedraw_bitmapgc);
  }

  mainw->framedraw_bitmapgc=gdk_gc_new (mainw->framedraw_bitmap);

  mainw->framedraw_image = gtk_image_new_from_pixmap (NULL, NULL);
  gtk_container_add (GTK_CONTAINER (frame), mainw->framedraw);
  gtk_container_add (GTK_CONTAINER (mainw->framedraw), mainw->framedraw_image);

  gtk_widget_show (mainw->framedraw);

  // load the start frame and show it
  load_framedraw_image();
  gtk_widget_show (mainw->framedraw_image);

  hbox27 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox27);
  gtk_box_pack_start (GTK_BOX (vbox27), hbox27, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox27), 10);

  spinbutton53_adj = gtk_adjustment_new (start, start, end, 1, 10, 0);
  spinbutton53 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton53_adj), 1, 0);
  gtk_widget_show (spinbutton53);
  gtk_box_pack_start (GTK_BOX (hbox27), spinbutton53, FALSE, FALSE, 0);

  label114 = gtk_label_new (_("    Frame"));
  gtk_widget_show (label114);
  gtk_box_pack_start (GTK_BOX (hbox27), label114, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label114), GTK_JUSTIFY_LEFT);
  if (palette->style&STYLE_1) {
    gtk_widget_modify_fg(label114, GTK_STATE_NORMAL, &palette->normal_fore);
  }

  mainw->framedraw_redraw = gtk_button_new_from_stock ("gtk-refresh");
  gtk_button_set_label (GTK_BUTTON (mainw->framedraw_redraw),_ ("Redraw"));
  gtk_box_pack_start (GTK_BOX (hbox27), mainw->framedraw_redraw, TRUE, FALSE, 0);

  framedraw_scale=gtk_hscale_new_with_range(0,cfile->frames,1);
  gtk_widget_show(framedraw_scale);
  gtk_box_pack_start (GTK_BOX (hbox27), framedraw_scale, TRUE, TRUE, 0);
  gtk_range_set_adjustment(GTK_RANGE(framedraw_scale),GTK_ADJUSTMENT(spinbutton53_adj));
  gtk_scale_set_draw_value(GTK_SCALE(framedraw_scale),FALSE);

  if (has_redraw_button==TRUE) {
    gtk_widget_show (mainw->framedraw_redraw);
  }
  else {
    gtk_widget_hide (mainw->framedraw_redraw);
  }

  gtk_widget_set_sensitive (mainw->framedraw_redraw,FALSE);

  g_signal_connect_after (GTK_OBJECT (spinbutton53), "value_changed",
			  G_CALLBACK (on_framedraw_spinbutton_changed),
			  NULL);

  // add mouse fn's so we can draw on frames
    g_signal_connect (GTK_OBJECT (mainw->framedraw), "motion_notify_event",
                      G_CALLBACK (on_framedraw_mouse_update),
                      (gpointer)(top));
    g_signal_connect (GTK_OBJECT (mainw->framedraw), "button_release_event",
                      G_CALLBACK (on_framedraw_mouse_reset),
                      (gpointer)(top));
    g_signal_connect (GTK_OBJECT (mainw->framedraw), "button_press_event",
                      G_CALLBACK (on_framedraw_mouse_start),
                      (gpointer)(top));


  if (mainw->framedraw_colourgc!=NULL) {
    g_object_unref (mainw->framedraw_colourgc);
    mainw->framedraw_colourgc=NULL;
  }

  // we have to make the toplevel window unresizable, so we can draw on frames
  gtk_window_set_resizable (GTK_WINDOW (gtk_widget_get_toplevel (mainw->framedraw)),FALSE);

}





void
on_framedraw_spinbutton_changed          (GtkSpinButton   *spinbutton,
					  gpointer         user_data)
{
  // update the single frame/framedraw preview
  mainw->framedraw_frame=gtk_spin_button_get_value_as_int(spinbutton);
  switch (mainw->framedraw_type) {
  case FD_TRIM_FRAMES:
    on_trim_frames_redraw(NULL, GINT_TO_POINTER (1));
    break;
  }
}




void 
load_framedraw_image(void) {
  // this is for the single frame framedraw widget
  // it should be called whenever mainw->framedraw_bitmap changes

  // it will load mainw->framedraw_frame into mainw->framedraw_pixmap
  // and combine this with mainw->framedraw_bitmap to create
  // mainw->framedraw_image

  // see for example on_trim_frames_redraw for how to combine with a mask

  GError *error=NULL;
  gchar buf[256];
  GdkPixbuf *pixbuf=NULL;
  GdkBitmap *dummy_bitmap;

  g_snprintf(buf,256,"%s/%s/%08d.jpg",prefs->tmpdir,cfile->handle,mainw->framedraw_frame);
  do {
    if (!(error==NULL)) {
      g_error_free(error);
      error=NULL;
    }
    pixbuf=gdk_pixbuf_new_from_file(buf,&error);
  } while (!(error==NULL));

  if (!(mainw->framedraw_orig_pixmap==NULL)) {
    gdk_pixmap_unref (mainw->framedraw_orig_pixmap);
  }

  // we just want the pixmap
  gdk_pixbuf_render_pixmap_and_mask (pixbuf, &mainw->framedraw_orig_pixmap, &dummy_bitmap, 128);
  if (!(dummy_bitmap==NULL)) {
    gdk_pixmap_unref (dummy_bitmap);
  }
  if (pixbuf!=NULL) {
    gdk_pixbuf_unref (pixbuf);
  }

  if (mainw->framedraw_colourgc==NULL) {
    mainw->framedraw_colourgc=gdk_gc_new (mainw->framedraw_orig_pixmap);
  }

  reload_framedraw_image();
}





void 
reload_framedraw_image(void) {
  // this will reset the pixmap to its virgin state, allow drawing, and then set
  // the gtk_image

  if (!(mainw->framedraw_copy_pixmap==NULL)) {
    gdk_pixmap_unref (mainw->framedraw_copy_pixmap);
  }

  mainw->framedraw_copy_pixmap=gdk_pixmap_copy (mainw->framedraw_orig_pixmap);
  // do any drawing on the copy_pixmap here
  gtk_image_set_from_pixmap(GTK_IMAGE(mainw->framedraw_image), mainw->framedraw_copy_pixmap, mainw->framedraw_bitmap);

}










// using these 3 functions, the user can draw on frames

gboolean
on_framedraw_mouse_start           (GtkWidget       *widget,
				    GdkEventButton  *event,
				    gpointer         top)
{
  // user clicked in the framedraw widget
  //  if (event->button==1); etc

  if (event->button==1) {
    gdk_window_get_pointer(GDK_WINDOW (mainw->framedraw->window), &xstart, &ystart, NULL);
    b1_held=TRUE;
  }
  return FALSE;
}

gboolean
on_framedraw_mouse_update           (GtkWidget       *widget,
				     GdkEventButton  *event,
				     gpointer         top)
{
  // pointer moved in the framedraw widget
  gint xcurrent,ycurrent;

  gdk_window_get_pointer(GDK_WINDOW (mainw->framedraw->window), &xcurrent, &ycurrent, NULL);

  switch (mainw->framedraw_type) {
  case FD_TRIM_FRAMES:
    if (b1_held) {
      reload_framedraw_image();
      if (mainw->fx1_bool==TRUE) {
	xstart=cfile->hsize-xcurrent;
	ystart=cfile->vsize-ycurrent;
      }
      draw_rect (&palette->light_yellow,xstart,ystart,xcurrent,ycurrent,FALSE);
      gtk_image_set_from_pixmap(GTK_IMAGE(mainw->framedraw_image), mainw->framedraw_copy_pixmap, mainw->framedraw_bitmap);
    }
    break;
  }
  return FALSE;
}


gboolean
on_framedraw_mouse_reset           (GtkWidget       *widget,
				    GdkEventButton  *event,
				    gpointer         user_data)
{
  gint xend,yend;
  gdk_window_get_pointer(GDK_WINDOW (mainw->framedraw->window), &xend, &yend, NULL);


// user released the mouse button in framedraw widget
  if (event->button==1) {
    b1_held=FALSE;
  }

  switch (mainw->framedraw_type) {
  case FD_TRIM_FRAMES:
    if (1) {
      _ftrimw *ftrimw=(_ftrimw *)user_data;
      if (xend>=cfile->hsize) xend=cfile->hsize-1;
      if (yend>=cfile->vsize) yend=cfile->vsize-1;

      gtk_spin_button_set_value(GTK_SPIN_BUTTON(ftrimw->spinbutton_x),MIN (xstart,xend));
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(ftrimw->spinbutton_y),MIN (ystart,yend));
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(ftrimw->spinbutton_width),ABS (xend-xstart));
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(ftrimw->spinbutton_height),ABS (yend-ystart));
    }
    break;
  }


  return FALSE;
}






// various drawing functions




void 
draw_rect (GdkColor *col, gint x1, gint y1, gint x2, gint y2, gboolean filled)
{
  gdk_gc_set_rgb_fg_color (GDK_GC (mainw->framedraw_colourgc),col);
  gdk_draw_rectangle (GDK_DRAWABLE (mainw->framedraw_copy_pixmap),GDK_GC (mainw->framedraw_colourgc), filled, MIN (x1,x2), MIN (y1,y2), ABS (x2-x1), ABS (y2-y1));
}
